<?php

namespace Drupal\cloudinary_media_library_widget\Plugin\Field\FieldWidget;

use Cloudinary\Api\Admin\AdminApi;
use Cloudinary\Configuration\Configuration;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\media\MediaInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\ConstraintViolationInterface;

/**
 * Plugin implementation of the 'cloudinary_media_library_widget' widget.
 *
 * @FieldWidget(
 *   id = "cloudinary_media_library_widget",
 *   label = @Translation("Cloudinary media library"),
 *   description = @Translation("Allows you to select cloudinary assets."),
 *   field_types = {
 *     "entity_reference"
 *   },
 *   multiple_values = FALSE,
 * )
 *
 * @internal
 *   Plugin classes are internal.
 */
class CloudinaryMediaLibraryWidget extends WidgetBase {

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The Cloudinary media manager.
   *
   * @var \Drupal\cloudinary_media_library_widget\CloudinaryMediaManagerInterface
   */
  protected $cloudinaryMediaManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->configFactory = $container->get('config.factory');
    $instance->cloudinaryMediaManager = $container->get('cloudinary_media_library_widget.manager.cloudinary_media');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'resource_type' => 'image',
    ] + parent::defaultSettings();
  }

  /**
   * Get resource type options.
   *
   * @return array
   *   List of options.
   */
  protected function getResourceTypeOptions() {
    $options = [];

    foreach ($this->cloudinaryMediaManager->getDefinitions() as $definition) {
      $options[$definition['resource_type']] = $definition['label'];
    }

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['resource_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Default resource'),
      '#options' => $this->getResourceTypeOptions(),
      '#default_value' => $this->getSetting('resource_type'),
      '#description' => $this->t('Select the default resource type preselected on Cloudinary media widget.'),
      '#required' => TRUE,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $options = $this->getResourceTypeOptions();

    $summary[] = $this->t('Cloudinary resource type: @resource', [
      '@resource' => $options[$this->getSetting('resource_type')],
    ]);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    try {
      Configuration::instance()->validate();
    }
    catch (\Exception $e) {
      $this->messenger()->addWarning($this->t('You have not provided a valid connection to your Cloudinary instance. Follow @link to set up a connection.', [
        '@link' => Link::createFromRoute($this->t('this link'), 'cloudinary_sdk.settings')->toString(),
      ]));
    }

    $referenced_entities = $items->referencedEntities();

    /** @var \Drupal\media\MediaInterface $referenced_entity */
    $referenced_entity = $referenced_entities[$delta] ?? NULL;

    $field_settings = $this->fieldDefinition->getSettings();

    // Only one target bundle is supported at the moment.
    $bundle = reset($field_settings['handler_settings']['target_bundles']);

    $element += [
      '#type' => 'textfield',
      '#default_value' => $referenced_entity ? $referenced_entity->id() : NULL,
      '#attributes' => ['class' => ['js-cloudinary-media--input']],
      '#attached' => [
        'library' => ['cloudinary_media_library_widget/widget'],
        'drupalSettings' => $this->getDrupalJavascriptSettings($bundle),
      ],
    ];

    $build = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['cloudinary-media'],
      ],
    ];

    $build['preview'] = [
      '#type' => 'inline_template',
      '#weight' => 1,
      '#template' => '<div class="js-cloudinary-media--preview">{{ preview }}</div>',
      '#context' => [
        'preview' => [],
      ],
    ];

    $resource_type = $this->getSetting('resource_type');
    $cloudinary_media_plugin = $this->cloudinaryMediaManager->createInstanceByType($resource_type);

    if ($referenced_entity instanceof MediaInterface) {
      $build['preview']['#context']['preview'] = $cloudinary_media_plugin->getPreview($referenced_entity);
    }

    $build['remove'] = [
      '#type' => 'inline_template',
      '#weight' => 2,
      '#template' => '<a href="#" class="js-cloudinary-media--remove">{{ label }}</a>',
      '#context' => [
        'label' => $this->t('Remove'),
      ],
    ];

    $build['target_id'] = $element;

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    return $field_definition->getSetting('target_type') === 'media';
  }

  /**
   * {@inheritdoc}
   */
  public function errorElement(array $element, ConstraintViolationInterface $error, array $form, FormStateInterface $form_state) {
    return $element['target_id'];
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as $key => $value) {
      if (empty($value['target_id'])) {
        unset($values[$key]);
      }
    }

    return $values;
  }

  /**
   * Get drupal javascript settings.
   *
   * @param string $bundle
   *   Media bundle.
   *
   * @return array
   *   Cloudinary js settings.
   */
  protected function getDrupalJavascriptSettings(string $bundle): array {
    $cardinality = $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();

    $sdk_settings = $this->configFactory->get('cloudinary_sdk.settings');
    $widget_settings = $this->configFactory->get('cloudinary_media_library_widget.settings');

    $settings = [
      'cloud_name' => $sdk_settings->get('cloudinary_sdk_cloud_name'),
      'api_key' => $sdk_settings->get('cloudinary_sdk_api_key'),
      'bundle' => $bundle,
      'multiple' => $cardinality != 1,
      'resource_type' => $this->getSetting('resource_type'),
      'use_saml' => $widget_settings->get('cloudinary_saml_auth'),
    ];

    $starting_folder = $widget_settings->get('cloudinary_starting_folder');

    if ($starting_folder !== '/') {
      $settings['starting_folder'] = $starting_folder;
    }

    if ($cardinality != FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED) {
      $settings['max_files'] = $cardinality;
    }

    return [
      'cloudinary_media_library_widget' => $settings,
    ];
  }

}
